@@ -10,35 +10,35 @@ from .. import pathtool
 from ..termcolor import tc
 if os.name == "nt":
     import win32serviceutil
-    
+
 class Service:
     def __init__ (self, name, working_dir, lockpath = None, win32service = None):
         self.name = name
         self.working_dir = working_dir
         self.lockpath = lockpath or working_dir
-        self.win32service = win32service        
+        self.win32service = win32service
         setproctitle and setproctitle (name)
         pathtool.mkdir (working_dir)
         if lockpath:
             pathtool.mkdir (lockpath)
         os.chdir (working_dir)
-        
+
     def start (self):
         if os.name == "nt":
             set_service_config (['start'])
-        else:    
-            from . import Daemonizer            
+        else:
+            from . import Daemonizer
             if not Daemonizer (self.working_dir, self.name, lockpath = self.lockpath).runAsDaemon ():
-                pid = daemon.status (self.lockpath, self.name)                
+                pid = daemon.status (self.lockpath, self.name)
                 print ("{} {}".format (tc.debug (self.name), tc.error ("[already running:{}]".format (pid))))
                 sys.exit ()
-            
+
     def stop (self):
-        if os.name == "nt":            
+        if os.name == "nt":
             set_service_config (['stop'])
-        else:    
+        else:
             daemon.kill (self.lockpath, self.name, True)
-    
+
     def status (self, verbose = True):
         pid = daemon.status (self.lockpath, self.name)
         if verbose:
@@ -47,7 +47,7 @@ class Service:
             else:
                 print ("{} {}".format (tc.debug (self.name), tc.secondary ("[stopped]")))
         return pid
-    
+
     def execute (self, cmd):
         if cmd == "stop":
             self.stop ()
@@ -58,36 +58,87 @@ class Service:
         elif cmd == "start":
             self.start ()
         elif cmd == "restart":
-            self.stop ()        
+            self.stop ()
             time.sleep (2)
             self.start ()
-        elif cmd == "install":    
+        elif cmd == "install":
             self.install ()
-        elif cmd == "update":    
+            return False
+        elif cmd == "update":
             self.update ()
-        elif cmd == "remove":    
+            return False
+        elif cmd == "remove":
             self.remove ()
+            return False
         else:
             raise AssertionError ('unknown command: %s' % cmd)
-                
+
         if os.name == "nt":
             return False
         return True
-    
+
     if os.name == "nt":
         def set_service_config (self, argv = []):
-            argv.insert (0, "")            
+            argv.insert (0, "")
             script = os.path.join (os.getcwd (), sys.argv [0])
             win32serviceutil.HandleCommandLine(self.win32service, "%s.%s" % (script [:-3], self.win32service.__name__), argv)
-                
+
         def install (self):
             self.set_service_config (['--startup', 'auto', 'install'])
-    
+
         def remove (self):
             self.set_service_config (['remove'])
-        
+
         def update (self):
             self.set_service_config (['update'])
 
+    else:
+        def install (self):
+            name = self.name.split ('/')[-1]
+            user = os.getenv("SUDO_USER") or 'nobody'
+            args = sys.argv [1:]
+            if args and hasattr (self, args [-1]):
+                args = args [:-1]
 
-            
\ No newline at end of file
+            script = SERVICE_TEMPLATE.format (
+                name = name.title (),
+                user = user,
+                executable = sys.executable,
+                serve = os.path.abspath (sys.argv [0]),
+                args = " ".join (args)
+            )
+            service_script = '/etc/systemd/system/{}.service'.format (name)
+            with open (service_script, 'w') as f:
+                f.write (script)
+            os.chmod (service_script, 0o755)
+
+            os.system ("systemctl daemon-reload")
+            print ("service {} installed".format (name))
+            print ("  sudo systemctl start {}".format (name))
+
+        def remove (self):
+            self.set_service_config (['remove'])
+
+        def update (self):
+            self.set_service_config (['update'])
+
+    uninstall = remove
+
+SERVICE_TEMPLATE = '''[Unit]
+Description=Skitai {name} Service
+
+[Service]
+Type=simple
+User={user}
+Group={user}
+StandardOutput=syslog+console
+StandardError=syslog+console
+Restart=on-failure
+WorkingDirectory=/var/tmp
+ExecStart={executable} {serve} {args}
+ExecStop=/bin/kill -s TERM $MAINPID
+ExecReload=/bin/kill -s HUP $MAINPID
+
+[Install]
+WantedBy=multi-user.target
+'''
